/* Copyright
   (c) 2008 bitSplit(tm) Enterprises and its licensors.
   This notice supersedes all notices below.
   -----------------------------------------------------------------------------------------
*/
//------------------------------------------------------------------------------------------
// Christopher M. Knox
// 2008/03/18
//------------------------------------------------------------------------------------------

#include "UUDefines.h"
#include "UUErrorCodes.h"
#include "UUTextParser.h"

//------------------------------------------------------------------------------------------
#if 0 
#pragma mark -
#pragma mark UUTextParser
#pragma mark -
#endif
//------------------------------------------------------------------------------------------

UUTextParser::UUTextParser(void)
{mInit(1); return;}

//------------------------------------------------------------------------------------------

UUTextParser::~UUTextParser(void)
{mDispose(1); return;}

//------------------------------------------------------------------------------------------

errcode UUTextParser::mInit(short decon)
{
	errcode	err = kUUERR_noErr;
	
	//if (! decon) err = UUBase::mInit(decon);
					
	//---
	// initialize members specific to the class
	
	strcpy(fClassName,"UUTextParser");
	
	fText = nil;
	fTextSize = 0;
	fCurPos = 0L;
	fAllocSize = 0;
	
	fFoundKey = 0;
	fAtLineStart = false;
	
	return err;
}

//------------------------------------------------------------------------------------------

errcode UUTextParser::mDispose(short decon)
{
	errcode	err = kUUERR_noErr;

	//---
	// dispose of members specific to the class
	
	mDisposeText();
	
	//---
	
	//if (! decon) err = UUBase::mDispose(decon);

	return err;
}

//----------------------------------------------------------------------------------------------

errcode UUTextParser::mError(errcode err)
{	
	string4K	msg, desc;
	
	msg[0]	= 0;
	desc[0]	= 0;
	
	if ((err)&&(err != kUUERR_userCanceled))
	{
        fprintf(stdout, "ERROR: %ld (%s) %s\n", err, msg, desc);
	}
	
	return err;
}

//------------------------------------------------------------------------------------------

errcode UUTextParser::mDisposeText(void)
{
	errcode	err = kUUERR_noErr;

	if (fText)
		FREE(fText);
	
	fText = nil;
	fTextSize = 0;
	fAllocSize = 0;
	fCurPos = 0L;

	return err;
}

//------------------------------------------------------------------------------------------

errcode UUTextParser::mSetText(const char *filePath)
{
	errcode		err = kUUERR_noErr;
	long8		textSize;
	FILE		*fp = nil;
	
	mDisposeText();
	
	fp = fopen(filePath, "rb" );
	if (! fp) {err = 1; goto finish;}

	fseek( fp, 0L, SEEK_END );
	textSize = ftell( fp );

	fText = CALLOC(textSize+4L);
	if (! fText) {err = 2; goto finish;}

	fseek( fp, 0L, SEEK_SET );
	fread(fText, 1, textSize, fp);
	
	fclose(fp);
	fp = nil;
	
	fText[textSize] = 0; // to be really really sure :-)
	fTextSize = fAllocSize = textSize;
	
finish:
	if (fp) fclose(fp);
	return err;
}


//------------------------------------------------------------------------------------------

errcode UUTextParser::mSetText(const char *text, long textSize)
{
	errcode		err = kUUERR_noErr;

	mDisposeText();
	
	if (! text)
	{
		if (textSize <= 0)
			goto finish;
		
		// just allocate
		
		fText = CALLOC(textSize+4L);
		if (! fText) {err = kUUERR_memFull; goto finish;}
		fTextSize = 0;
		fAllocSize = textSize;
	}
	else
	{
		if (textSize <= 0)
			textSize = strlen(text);
			
		fText = CALLOC(textSize+4L);
		if (! fText) {err = kUUERR_memFull; goto finish;}
		
		MEMCPY(fText,text,textSize);
		fText[textSize] = 0; // to be really really sure :-)
		fTextSize = fAllocSize = textSize;
	}
	
finish:
	return err;
}

//------------------------------------------------------------------------------------------

errcode UUTextParser::mWriteTextToFile(const char *inFilePath)
{
	errcode		err = kUUERR_noErr;
	FILE		*fp = nil;
	
	// write the text to the specified file
	
	fp = fopen(inFilePath, "wb" );
	if (! fp) {err = 1; goto finish;}

	if ((fText)&&(fTextSize > 0))
		fwrite(fText, 1, fTextSize, fp);

finish:
	if (fp) fclose(fp);
	return err;
}

//------------------------------------------------------------------------------------------

errcode UUTextParser::mSetPos(long newPos)
{
	errcode		err = kUUERR_noErr;
	
	if ((newPos < 0)||(newPos >= fTextSize))
	{err = kUUERR_invalidIndex; goto finish;}

	fCurPos = newPos;
	
finish:
	return err;
}

//------------------------------------------------------------------------------------------

char *UUTextParser::mFindString(const char *str)
{
	char		*retpC = nil, *pC;
	
	// find 'str' starting at the current position
	// return a pointer to its start (or nil if not found)
	// advance to just past 'str'
	
	if ((! str)
		||(! fText)
		||(fCurPos >= fTextSize)
	)
	goto finish;
	
	if (fAtLineStart) // CHK 2006/10/14
	{
		int		curPos, tpos;
		
		curPos = fCurPos;
		
		while(1)
		{
			pC = fText+curPos;
			retpC = strstr(pC,str);
			if (! retpC) // not found
				break;
				
			tpos = (ulong) retpC - (ulong) fText;
			if (! tpos) // start of text
				break;
			
			if ((retpC[-1] == '\n')||(retpC[-1] == '\r'))
				break; // at line start
			
			retpC = nil;
			
			curPos += strlen(str); // try again
			if (curPos >= fTextSize)
				break;
		};
	}
	else
	{
		pC = fText+fCurPos;
		retpC = strstr(pC,str);
	}
	
	if (retpC) // found it, advance past the string
	{
		fCurPos = (ulong) retpC - (ulong) fText;
		fCurPos += strlen(str);
	}
	
finish:
	fAtLineStart = false; // CHK 2006/10/14
	return retpC;
}

//------------------------------------------------------------------------------------------

errcode UUTextParser::mGetStringBetween(const char *str1, const char *str2, char *foundStr, long maxLen)
{
	errcode			err = kUUERR_noErr;
	UUTextParser	foundTP;
	int				len;
	
	foundStr[0] = 0;

	err = mGetTextBetween(str1, str2, &foundTP);
	if (err) goto finish;
	
	if (foundTP.fText)
	{
		len = MIN(foundTP.fTextSize, maxLen);
		MEMCPY(foundStr, foundTP.fText, len);
		foundStr[len] = 0;
	}
	
finish:
	return err;
}

//------------------------------------------------------------------------------------------

errcode UUTextParser::mGetTextBetween(const char *str1, const char *str2, UUTextParser *foundTP)
{
	errcode		err = kUUERR_noErr;
	char		*spC, *epC;
	long		len;
	
	// return the string between 'str1' and 'str2' up to maxLen (null terminated)
	// start at the current position
	// advance past str2 if found
	
	fTempStartPos = -1;
	
	if (! foundTP)
	{err = kUUERR_invalidObject; goto finish;}
	
	foundTP->mDisposeText();
	
	if (! fText)
	{err = kUUERR_nilPtr; goto finish;}
		
	if (str1) // CHK 2005/09/16
	{
		spC = mFindString(str1);
		if (! spC) {err = kUUERR_notFound; goto finish;}
	}
	
	fTempStartPos = fCurPos;
	
	spC = fText+fCurPos; // point to start of string we want
	
	epC = mFindString(str2);
	if (! epC) {err = kUUERR_notFound; goto finish;}
	
	len = (ulong) epC - (ulong) spC;
	
	foundTP->mAddText(spC, len);
	
finish:
	return err;
}

//------------------------------------------------------------------------------------------

errcode UUTextParser::mReplaceString(const char *findStr,const char *replaceWith)
{
	errcode		err = kUUERR_noErr;
	char		*spC,*dpC,*newText = nil;
	long		newLen,flen,rlen,tlen;
	
	// starts at the current position and looks for 'findStr'
	// if found it will be replaced by 'replaceWith'
	// will advance past 'replaceWith' and adjust text size
	// note that 'fText' may get reallocated
	
	if (! findStr)
	{err = kUUERR_nilPtr; goto finish;}
	if (! replaceWith)
	{err = kUUERR_nilPtr; goto finish;}
	
	if (! fText)
	{err = kUUERR_nilPtr; goto finish;}

	spC = mFindString(findStr);
	if (! spC) {err = kUUERR_notFound; goto finish;}
	
	flen = strlen(findStr);
	rlen = strlen(replaceWith);
	
	if (flen == rlen)   // same length, just copy the new string
	{
		spC = fText+fCurPos-flen; // point to start of the string
		MEMCPY(spC,replaceWith,rlen);
	}
	else // allocate a new text buffer with the proper size
	{
		newLen = fTextSize - flen + rlen;
		newText = CALLOC(newLen+4);
		newText[newLen]	= 0;	
		if (! newText) 
		{err = kUUERR_memFull; goto finish;}
		
		// copy up to the string
		tlen = fCurPos-flen;
		dpC = newText;
		MEMCPY(dpC,fText,tlen);

		// copy the string
		dpC += tlen;
		MEMCPY(dpC,replaceWith,rlen);

		// copy remaining text after the string
		dpC += rlen;
		tlen = fTextSize-fCurPos;
		spC = fText+fCurPos;
		MEMCPY(dpC,spC,tlen);
		
		// replace pointer
		FREE(fText);
		fText = newText;
		newText = nil;
		
		fTextSize = fAllocSize = newLen;
		fCurPos += -flen+rlen; // point past the new string
	}
	
finish:
	if (newText) FREE(newText);
	return err;
}

//------------------------------------------------------------------------------------------

errcode UUTextParser::mAddLF(void)
{
	errcode		err = kUUERR_noErr;
	long		deltaAlloc = kUU_1KB, newLen, len = 2;
	char		*dpC, *newText = nil;
	
	newLen = fTextSize + len;
	
	if (newLen >= fAllocSize)
	{
		newText = CALLOC(newLen+deltaAlloc);
		
		if (! newText) 
		{err = kUUERR_memFull; goto finish;}
		
		fAllocSize = newLen+deltaAlloc;
		
		if (fText)
		{
			MEMCPY(newText, fText, fTextSize);
			FREE(fText);
		}
		
		fText = newText;
		newText = nil;
	}

	dpC = fText+fTextSize;
	*dpC++ = '\n';
	*dpC++ = 0;
	++fTextSize;
	
finish:
	if (newText) FREE(newText);
	return err;
}

//------------------------------------------------------------------------------------------

errcode UUTextParser::mAddString(const char *text)
{
	return mAddText(text, strlen(text));
}

//------------------------------------------------------------------------------------------

errcode UUTextParser::mAddText(const char *text, long len)
{
	errcode		err = kUUERR_noErr;
	long		deltaAlloc = kUU_1KB, newLen;
	char		*dpC, *newText = nil;
	
	if (! text)
		goto finish;
	if (len <= 0)
		goto finish;
			
	newLen = fTextSize + len;
	
	if (newLen >= fAllocSize)
	{
		newText = CALLOC(newLen+deltaAlloc);
		
		if (! newText) 
		{err = kUUERR_memFull; goto finish;}
		
		fAllocSize = newLen+deltaAlloc;
		
		if (fText)
		{
			MEMCPY(newText,fText,fTextSize);
			FREE(fText);
		}
		
		fText = newText;
		newText = nil;
	}

	dpC = fText+fTextSize;
	MEMCPY(dpC,text,len);
	fTextSize += len;
	fText[fTextSize] = 0;
	
finish:
	if (newText) FREE(newText);
	return err;
}

//------------------------------------------------------------------------------------------

double UUTextParser::mGetNumber(const char *key, double defNum)
{
	double		retVal;
	string256   str1,str2,foundStr;
	
	fFoundKey = 0;
	retVal = defNum;
	
	if (! strstr(key, " = "))	sprintf(str1, "%s = ", key);
	else						strcpy(str1, key);

	sprintf(str2,"\n");
	
	mSetPos(0L);
	if (mGetStringBetween(str1,str2,foundStr,sizeof(foundStr)-1))
		goto finish;

	retVal = strtod(foundStr,nil);
	fFoundKey = 1;
	
finish:
	return retVal;
}

//------------------------------------------------------------------------------------------

char *UUTextParser::mGetString(const char *key, const char *defStr)
{
	char		*retVal = fTempStr;
	string256   str1, str2;
	string8K	foundStr;
	
	fFoundKey = 0;
	strncpy(retVal, defStr, sizeof(fTempStr)-1);
	
	if (! strstr(key, " = "))	sprintf(str1, "%s = ", key);
	else						strcpy(str1, key);

	sprintf(str2,"\n");
	
	mSetPos(0L);
	if (mGetStringBetween(str1, str2, foundStr, sizeof(foundStr)-1))
		goto finish;
	
	strncpy(retVal, foundStr, sizeof(fTempStr)-1);
	fFoundKey = 1;
	
finish:
	return retVal;
}

//------------------------------------------------------------------------------------------
